import gtk


class XPNToolBar:
    def get_widget(self):
        return self.toolbar

    def show(self):
        self.toolbar.show_all()

    def get_buttons(self):
        return self.buttons

    def insert(self,name,icon_name,shortname,tooltip):
        icon=gtk.Image()
        icon.set_from_file("pixmaps/"+icon_name)
        toolbutton=gtk.ToolButton(icon,name)
        toolbutton.set_tooltip(self.tips,tooltip,tooltip)
        self.toolbar.insert(toolbutton,-1)
        self.buttons[shortname]=toolbutton
        self.toolbar.set_tooltips(True)

    def append_space(self):
        separator=gtk.SeparatorToolItem()
        self.toolbar.insert(separator,-1)

    def append_label(self,label_text):
        toolitem=gtk.ToolItem()
        label=gtk.Label(label_text)
        toolitem.add(label)
        self.toolbar.insert(toolitem,-1)

    def __init__(self):

        self.buttons={}

        #Toolbar
        self.toolbar=gtk.Toolbar()
        self.toolbar.set_orientation(gtk.ORIENTATION_HORIZONTAL)
        self.toolbar.set_style(gtk.TOOLBAR_ICONS)
        self.toolbar.set_tooltips(True)
        self.tips=gtk.Tooltips()

class MainToolBar(XPNToolBar):
    def __init__(self):
        XPNToolBar.__init__(self)
        #Groups
        self.append_label(_("Groups:"))
        self.insert(_("Groups"),"groups.xpm","groups",_("Manage Groups"))
        self.insert(_("Headers"),"receive_headers.xpm","gethdrs",_("Get New Headers in Subscribed Groups"))
        self.insert(_("Bodies"),"receive_bodies.xpm","getbodies",_("Get Marked Article Bodies in Subscribed Groups"))
        self.insert(_("Mark"),"mark.xpm","mark",_("Mark Selected Groups as Read"))
        self.insert(_("Mark All"),"mark_all.xpm","markall",_("Mark All Groups as Read"))
        self.append_space()


        #Write
        self.append_label(_("Write:"))
        self.insert(_("Post"),"post.xpm","post",_("Post New Article"))
        self.insert(_("Follow-Up"),"followup.xpm","followup",_("Follow-Up To Newsgroup"))
        self.insert(_("Reply"),"reply.xpm","reply",_("Reply by Mail"))
        self.insert(_("Open Outbox"),"outbox.xpm","outbox_manager",_("Open Outbox Manager"))

        self.append_space()


        #Read
        self.append_label(_("Read:"))
        self.insert(_("Previous"),"previous.xpm","previous",_("Read Previous Article"))
        self.insert(_("Next"),"next.xpm","next",_("Read Next Article"))
        self.insert(_("Next Unread"),"next_unread.xpm","next_unread",_("Read Next Unread Article"))
        self.insert(_("Rot13"),"rot13.xpm","rot13",_("ROT13 Selected Text"))
        self.append_space()


        #Threads
        self.append_label(_("Threads:"))
        self.insert(_("Expand"),"expand.xpm","expand",_("Expand All"))
        self.insert(_("Collapse"),"collapse.xpm","collapse",_("Collapse All"))
        self.append_space()


        #Conf
        self.append_label(_("Configure:"))
        self.insert(_("Score"),"score.xpm","score",_("Edit Scoring Rules"))
        self.insert(_("Preferences"),"conf.xpm","conf",_("Preferences"))

        self.tips.enable()


class EditToolBar(XPNToolBar):
    def __init__(self):
        XPNToolBar.__init__(self)
        #Groups
        self.append_label(_("Send:"))
        self.insert(_("Send"),"send.xpm","send",_("Send Article"))
        self.insert(_("Send Later"),"send_queue.xpm","send_later",_("Send Article Later"))
        self.insert(_("Save as Draft"),"save.xpm","save_draft",_("Save Article as Draft"))
        self.insert(_("Discard"),"discard.xpm","discard",_("Discard Article"))
        self.append_space()

        #Edit
        self.append_label(_("Edit:"))
        self.insert(_("Rot13"),"rot13.xpm","rot13",_("ROT13 Selected Text"))
        self.insert(_("Spoiler Char"),"spoiler_char.xpm","spoiler",_("Insert Spoiler Char"))
        self.insert(_("Editor"),"editor.xpm","editor",_("Launch External Editor"))

        self.tips.enable()


class OutboxToolBar(XPNToolBar):
    def __init__(self):
        XPNToolBar.__init__(self)
        #Groups
        self.append_label(_("Send:"))
        self.insert(_("Send Articles"),"send_queued_art.xpm","send_article",_("Send Queued Articles"))
        self.insert(_("Send Mails"),"send_queued_mail.xpm","send_mail",_("Send Queued Mails"))
        self.append_space()

        #Edit
        self.append_label(_("Edit:"))
        self.insert(_("Edit"),"post.xpm","edit",_("Edit Article/Mail"))
        self.insert(_("Delete"),"delete.xpm","delete",_("Delete Article/Mail"))


        self.tips.enable()
